@@ -1,31 +1,14 @@
 #
-# $Id: Constants.pm,v 0.40 1999/07/15 18:26:18 dankogai Exp dankogai $
+# $Id: Constants.pm,v 0.56 1999/07/23 16:44:23 dankogai Exp $
 #
 
-=head1 NAME
-
-Jcode::Constants -- Constants used in Jcode
-
-=head1 DESCRIPTION
-
-This module is for internal use by Jcode modules.
-
-=head1 COPYRIGHT
-
-Copyright 1999 Dan Kogai <dankogai@dan.co.jp>
-
-This library is free software; you can redistribute it
-and/or modify it under the same terms as Perl itself.
-
-=cut
-
 package Jcode::Constants;
 
 use strict;
 use vars qw($RCSID $VERSION);
 
-$RCSID = q$Id: Constants.pm,v 0.40 1999/07/15 18:26:18 dankogai Exp dankogai $;
-$VERSION = do { my @r = (q$Revision: 0.40 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+$RCSID = q$Id: Constants.pm,v 0.56 1999/07/23 16:44:23 dankogai Exp $;
+$VERSION = do { my @r = (q$Revision: 0.56 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
 
 use Carp;
 
@@ -34,7 +17,7 @@ BEGIN {
     use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
     @ISA         = qw(Exporter);
     @EXPORT      = qw();
-    @EXPORT_OK   = qw(%CHARCODE %ESC %RE);
+    @EXPORT_OK   = qw(&_max &_mkbuf %CHARCODE %ESC %RE);
     %EXPORT_TAGS = ( 'all' => [ @EXPORT_OK, @EXPORT ] );
 }
 
@@ -62,6 +45,7 @@ my %_0208 = (
 
 %RE =
     (
+     ASCII     => '[\x00-\xff]',
      BIN       => '[\x00-\x06\x7f\xff]',
      EUC_0212  => '\x8f[\xa1-\xfe][\xa1-\xfe]',
      EUC_C     => '[\xa1-\xfe][\xa1-\xfe]',
@@ -80,7 +64,7 @@ my %_0208 = (
 
 # Make buffer when and only when necessary
 
-sub Jcode::_mkbuf {
+sub _mkbuf {
     my $thingy = shift;
     if (ref $thingy){
 	return $thingy;
@@ -91,7 +75,7 @@ sub Jcode::_mkbuf {
     }
 }
 
-sub Jcode::_max {
+sub _max {
     my $result = shift;
     for my $n (@_){
 	$result = $n if $n > $result;
